@@ -1,12 +1,12 @@
-# -*- coding: utf-8 -*-
-"""
-Created on Sat Jul 20 11:40:41 2019
 
-@author: johnmount
+"""
+Utility functions for vtreat
 """
 
 import math
+import re
 import statistics
+from typing import Iterable, List
 
 import hashlib
 
@@ -18,22 +18,46 @@ import vtreat.stats_utils
 
 
 def safe_to_numeric_array(x):
-    # work around https://github.com/WinVector/pyvtreat/issues/7
-    # noinspection PyTypeChecker
-    return numpy.asarray(pandas.Series(x) + 0.0, dtype=float)
+    """
+    Convert array to numeric. Note, will parse strings (due to numpy)!
+
+    :param x: array to process
+    :return: numeric array
+    """
+    # note will parse strings!
+    return numpy.asarray(x, dtype=float)
 
 
-def can_convert_v_to_numeric(x):
-    """check if non-empty vector can convert to numeric"""
+def can_convert_v_to_numeric(x) -> bool:
+    """
+    check if non-empty vector can convert to numeric
+
+    :param x:
+    :return: True if can convert to numeric, false otherwise (no string parsing).
+    """
+    x = numpy.asarray(x)
+    not_bad = numpy.logical_not(pandas.isnull(x))
+    n_not_bad = numpy.sum(not_bad)
+    if n_not_bad < 1:
+        return True
     try:
-        numpy.asarray(x + 0.0, dtype=float)
+        numpy.asarray(
+            x[not_bad] + 0, dtype=float
+        )  # +0 prevents string parse, leaving strings as strings
         return True
     except TypeError:
         return False
+    except ValueError:
+        return False
 
 
 def is_bad(x):
-    """ for numeric vector x, return logical vector of positions that are null, NaN, infinite"""
+    """
+    For numeric vector x, return logical vector of positions that are null, NaN, infinite.
+
+    :param x:
+    :return:
+    """
     if can_convert_v_to_numeric(x):
         x = safe_to_numeric_array(x)
         return numpy.logical_or(
@@ -42,17 +66,23 @@ def is_bad(x):
     return pandas.isnull(x)
 
 
-def numeric_has_range(x):
+def numeric_has_range(x) -> bool:
+    """
+    Check if a numeric vector has numeric range.
+
+    :param x: vector to check
+    :return: True if max > min values in vector, else False.
+    """
     x = safe_to_numeric_array(x)
     not_bad = numpy.logical_not(is_bad(x))
-    n_not_bad = sum(not_bad)
+    n_not_bad = numpy.sum(not_bad)
     if n_not_bad < 2:
         return False
     x = x[not_bad]
     return numpy.max(x) > numpy.min(x)
 
 
-def summarize_column(x, *, fn=numpy.mean):
+def summarize_column(x, *, fn=numpy.mean) -> float:
     """
     Summarize column to a non-missing scalar.
 
@@ -63,7 +93,7 @@ def summarize_column(x, *, fn=numpy.mean):
 
     x = safe_to_numeric_array(x)
     not_bad = numpy.logical_not(is_bad(x))
-    n_not_bad = sum(not_bad)
+    n_not_bad = numpy.sum(not_bad)
     if n_not_bad < 1:
         return 0.0
     x = x[not_bad]
@@ -78,7 +108,7 @@ def characterize_numeric(x):
 
     x = safe_to_numeric_array(x)
     not_bad = numpy.logical_not(is_bad(x))
-    n_not_bad = sum(not_bad)
+    n_not_bad = numpy.sum(not_bad)
     n = len(x)
     if n_not_bad <= 0:
         return {
@@ -107,10 +137,10 @@ def characterize_numeric(x):
 def get_unique_value_count(x):
     """compute how many unique values in list-x"""
     if len(x) <= 1:
-        return(len(x))
-    p = pandas.DataFrame({'x': x, 'o': 1})
-    s = p.groupby('x').sum()  # drops None
-    return(max(pandas.isnull(x)) + s.shape[0])
+        return len(x)
+    p = pandas.DataFrame({"x": x, "o": 1})
+    s = p.groupby("x").sum()  # drops None
+    return numpy.max(pandas.isnull(x)) + s.shape[0]
 
 
 def grouped_by_x_statistics(x, y):
@@ -137,7 +167,7 @@ def grouped_by_x_statistics(x, y):
     sf["_var"] = sf["_var"] / (sf["_ni"] - 1) + eps
     avg_var = 0
     bad_vars = is_bad(sf["_var"])
-    if sum(bad_vars) < len(sf["_var"]):
+    if numpy.sum(bad_vars) < len(sf["_var"]):
         avg_var = numpy.nanmean(sf["_var"])
     sf.loc[bad_vars, "_var"] = avg_var
     if sf.shape[0] > 1:
@@ -156,9 +186,7 @@ def grouped_by_x_statistics(x, y):
     return sf
 
 
-def score_variables(cross_frame, variables, outcome,
-                    *,
-                    is_classification=False):
+def score_variables(cross_frame, variables, outcome, *, is_classification=False):
     """score the linear relation of variables to outcome"""
 
     if len(variables) <= 0:
@@ -169,13 +197,16 @@ def score_variables(cross_frame, variables, outcome,
     outcome = safe_to_numeric_array(outcome)
 
     def f(v):
+        """summarizing fn"""
         col = cross_frame[v]
         col = safe_to_numeric_array(col)
-        if (n > 2) and \
-                (numpy.max(col) > numpy.min(col)) and \
-                (numpy.max(outcome) > numpy.min(outcome)):
+        if (
+            (n > 2)
+            and (numpy.max(col) > numpy.min(col))
+            and (numpy.max(outcome) > numpy.min(outcome))
+        ):
             cor, sig = vtreat.stats_utils.our_corr_score(y_true=outcome, y_pred=col)
-            r2 = cor**2
+            r2 = cor ** 2
             if is_classification:
                 r2, sig = vtreat.stats_utils.our_pseudo_R2(y_true=outcome, y_pred=col)
             sfi = pandas.DataFrame(
@@ -224,32 +255,55 @@ def check_matching_numeric_frames(*, res, expect, tol=1.0e-4):
         assert numpy.max(numpy.abs(ec - rc)) <= tol
 
 
-def unique_itmes_in_order(lst):
+def unique_items_in_order(x) -> list:
+    """
+    Return de-duplicated list of items in order they are in supplied array.
+
+    :param x: vector to inspect
+    :return: list
+    """
     ret = []
-    if lst is not None:
+    if x is not None:
         seen = set()
-        for item in lst:
+        for item in x:
             if item not in seen:
                 ret.append(item)
                 seen.add(item)
     return ret
 
 
-def clean_string(strng):
-    mp = {'<': '_lt_',
-          '>': '_gt_',
-          '[': '_osq_',
-          ']': '_csq_',
-          '(': '_op_',
-          ')': '_cp_',
-          '.': '_',
-          }
+def clean_string(s: str) -> str:
+    """
+    Replace common symbols with column-name safe alternatives.
+
+    :param s: incoming string
+    :return: string
+    """
+    mp = {
+        "<": "_lt_",
+        ">": "_gt_",
+        "[": "_osq_",
+        "]": "_csq_",
+        "(": "_op_",
+        ")": "_cp_",
+        ".": "_",
+        " ": "_",
+    }
+    s = s.strip()
+    s = re.sub(r'\s+', ' ', s)
     for (k, v) in mp.items():
-        strng = strng.replace(k, v)
-    return strng
+        s = s.replace(k, v)
+    return s
 
 
-def build_level_codes(incoming_column_name, levels):
+def build_level_codes(incoming_column_name: str, levels: Iterable) -> List[str]:
+    """
+    Pick level names for a set of levels.
+
+    :param incoming_column_name:
+    :param levels:
+    :return:
+    """
     levels = [str(lev) for lev in levels]
     levels = [incoming_column_name + "_lev_" + clean_string(lev) for lev in levels]
     if len(set(levels)) != len(levels):
@@ -257,5 +311,11 @@ def build_level_codes(incoming_column_name, levels):
     return levels
 
 
-def hash_data_frame(d):
+def hash_data_frame(d: pandas.DataFrame) -> str:
+    """
+    Get a hash code representing a data frame.
+
+    :param d: data frame
+    :return: hash code as a string
+    """
     return hashlib.sha256(pandas.util.hash_pandas_object(d).values).hexdigest()
