@@ -1,4 +1,10 @@
+
+"""
+Define user visible vtreat API.
+"""
+
 import warnings
+from typing import Any, Dict, Iterable, Optional
 
 import pandas
 import numpy
@@ -8,8 +14,13 @@ import vtreat.util
 import vtreat.cross_plan
 
 
-def vtreat_parameters(user_params=None):
-    """build a vtreat parameters dictionary, adding in user choices"""
+def vtreat_parameters(user_params: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
+    """
+    build a vtreat parameters dictionary, adding in user choices
+
+    :param user_params: dictionary of user choices
+    :return: dictionary of user and default choices
+    """
 
     params = {
         "use_hierarchical_estimate": True,
@@ -32,9 +43,7 @@ def vtreat_parameters(user_params=None):
         "check_for_duplicate_frames": True,
         "error_on_duplicate_frames": False,
         "retain_cross_plan": True,
-        "tunable_params": [
-            "indicator_min_fraction"
-            ],
+        "tunable_params": ["indicator_min_fraction"],
     }
     pkeys = set(params.keys())
     if user_params is not None:
@@ -50,8 +59,13 @@ def vtreat_parameters(user_params=None):
     return params
 
 
-def unsupervised_parameters(user_params=None):
-    """build a vtreat parameters dictionary for unsupervised tasks, adding in user choices"""
+def unsupervised_parameters(user_params: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
+    """
+    build a vtreat parameters dictionary for unsupervised tasks, adding in user choices
+
+    :param user_params: dictionary of user choices
+    :return: dictionary of user and default choices
+    """
 
     params = {
         "coders": {
@@ -64,9 +78,7 @@ def unsupervised_parameters(user_params=None):
         "user_transforms": [],
         "sparse_indicators": False,
         "missingness_imputation": numpy.mean,
-        "tunable_params": [
-            "indicator_min_fraction"
-        ],
+        "tunable_params": ["indicator_min_fraction"],
     }
     pkeys = set(params.keys())
     if user_params is not None:
@@ -84,18 +96,19 @@ class NumericOutcomeTreatment(vtreat_impl.VariableTreatment):
     """manage a treatment plan for a numeric outcome (regression)"""
 
     def __init__(
-            self, *,
-            var_list=None,
-            outcome_name=None,
-            cols_to_copy=None,
-            params=None,
-            imputation_map=None,
+        self,
+        *,
+        var_list: Optional[Iterable[str]] = None,
+        outcome_name: Optional[str] = None,
+        cols_to_copy: Optional[Iterable[str]] = None,
+        params: Optional[Dict[str, Any]] = None,
+        imputation_map: Optional[Dict[str, Any]] = None,
     ):
         """
 
-         :param var_list: list or touple of column names
+         :param var_list: list or tuple of column names, if empty all non outcome and copy columns are used
          :param outcome_name: name of column containing dependent variable
-         :param cols_to_copy: list or touple of column names
+         :param cols_to_copy: list or tuple of column names
          :param params: vtreat.vtreat_parameters()
          :param imputation_map: map of column names to custom missing imputation values or functions
         """
@@ -109,25 +122,46 @@ class NumericOutcomeTreatment(vtreat_impl.VariableTreatment):
             imputation_map=imputation_map,
         )
 
-    def merge_params(self, p):
+    def merge_params(self, p: Optional[Dict[str, Any]]) -> Dict[str, Any]:
+        """
+        Merge user parameters, returns new parameters does not alter object.
+
+        :param p:
+        :return: merged parameters
+        """
+
         return vtreat_parameters(p)
 
     # noinspection PyPep8Naming
     def transform(self, X):
+        """
+        Apply transform to data.
+
+        :param X: data
+        :return: transformed data
+        """
+
         X, orig_type = vtreat_impl.ready_data_frame(X)
         self.check_column_names(X.columns)
         if self.last_fit_x_id_ is None:
             raise ValueError("called transform on not yet fit treatment")
-        if self.params_['check_for_duplicate_frames'] and (self.last_fit_x_id_ == vtreat.util.hash_data_frame(X)):
+        if self.params_["check_for_duplicate_frames"] and (
+            self.last_fit_x_id_ == vtreat.util.hash_data_frame(X)
+        ):
             if self.params_["error_on_duplicate_frames"]:
                 raise ValueError(
-                    "possibly called transform on same data used to fit\n" +
-                    "(this causes over-fit, please use fit_transform() instead)")
+                    "possibly called transform on same data used to fit\n"
+                    + "(this causes over-fit, please use fit_transform() instead)"
+                )
             warnings.warn(
-                "possibly called transform on same data used to fit\n" +
-                "(this causes over-fit, please use fit_transform() instead)")
+                "possibly called transform on same data used to fit\n"
+                + "(this causes over-fit, please use fit_transform() instead)"
+            )
         res = vtreat_impl.pre_prep_frame(
-            X, col_list=self.var_list_, cols_to_copy=self.cols_to_copy_, cat_cols=self.plan_['cat_list']
+            X,
+            col_list=self.plan_["num_list"] + self.plan_["cat_list"],
+            cols_to_copy=self.cols_to_copy_,
+            cat_cols=self.plan_["cat_list"],
         )
         res = vtreat_impl.perform_transform(x=res, transform=self, params=self.params_)
         res = vtreat_impl.limit_to_appropriate_columns(res=res, transform=self)
@@ -137,6 +171,15 @@ class NumericOutcomeTreatment(vtreat_impl.VariableTreatment):
 
     # noinspection PyPep8Naming
     def fit_transform(self, X, y=None, **fit_params):
+        """
+        fit_transform data, this is the way to fit with cross methods.
+
+        :param X: explanatory values
+        :param y: dependent values
+        :param fit_params:
+        :return: transformed data
+        """
+
         X, orig_type = vtreat_impl.ready_data_frame(X)
         self.check_column_names(X.columns)
         if y is None:
@@ -147,7 +190,9 @@ class NumericOutcomeTreatment(vtreat_impl.VariableTreatment):
             y = numpy.asarray(y)
             if (self.outcome_name_ is not None) and (self.outcome_name_ in X.columns):
                 if not numpy.all(X[self.outcome_name_] == y):
-                    raise ValueError(".fit_transform(X, y) called with y != X[outcome_name]")
+                    raise ValueError(
+                        ".fit_transform(X, y) called with y != X[outcome_name]"
+                    )
         if not X.shape[0] == len(y):
             raise ValueError("X.shape[0] should equal len(y)")
         y = vtreat.util.safe_to_numeric_array(y)
@@ -157,7 +202,7 @@ class NumericOutcomeTreatment(vtreat_impl.VariableTreatment):
             raise ValueError("y does not vary")
         cross_rows = None
         cross_plan = None
-        if self.params_['retain_cross_plan']:
+        if self.params_["retain_cross_plan"]:
             cross_rows = self.cross_rows_
             cross_plan = self.cross_plan_
         self.clear()
@@ -179,24 +224,25 @@ class NumericOutcomeTreatment(vtreat_impl.VariableTreatment):
             params=self.params_,
             imputation_map=self.imputation_map_,
         )
-        res = vtreat_impl.perform_transform(x=X, transform=self, params=self.params_)
+        cross_frame = vtreat_impl.perform_transform(x=X, transform=self, params=self.params_)
         if (cross_plan is None) or (cross_rows != X.shape[0]):
             if cross_plan is not None:
-                warnings.warn("Number of rows different than previous fit with retain_cross_plan==True")
+                warnings.warn(
+                    "Number of rows different than previous fit with retain_cross_plan==True"
+                )
             cross_plan = self.params_["cross_validation_plan"].split_plan(
-                n_rows=X.shape[0], k_folds=self.params_["cross_validation_k"], data=X, y=y
+                n_rows=X.shape[0],
+                k_folds=self.params_["cross_validation_k"],
+                data=X,
+                y=y,
             )
             cross_rows = X.shape[0]
         # patch in cross-frame versions of complex columns such as impact
-        cross_frame = vtreat_impl.cross_patch_refit_y_aware_cols(
-            x=X, y=y, res=res, plan=self.plan_, cross_plan=cross_plan
+        vtreat_impl.cross_patch_refit_y_aware_cols(
+            x=X, y=y, res=cross_frame, plan=self.plan_, cross_plan=cross_plan
         )
-        cross_frame = vtreat_impl.cross_patch_user_y_aware_cols(
-            x=cross_frame,
-            y=y,
-            res=res,
-            params=self.params_,
-            cross_plan=cross_plan,
+        vtreat_impl.cross_patch_user_y_aware_cols(
+            x=cross_frame, y=y, res=cross_frame, params=self.params_, cross_plan=cross_plan,
         )
         # use cross_frame to compute variable effects
         self.score_frame_ = vtreat_impl.score_plan_variables(
@@ -204,17 +250,29 @@ class NumericOutcomeTreatment(vtreat_impl.VariableTreatment):
             outcome=y,
             plan=self.plan_,
             params=self.params_,
-            is_classification=False
+            is_classification=False,
         )
-        if ("filter_to_recommended" in self.params_.keys()) and self.params_["filter_to_recommended"]:
+        if ("filter_to_recommended" in self.params_.keys()) and self.params_[
+            "filter_to_recommended"
+        ]:
             self.set_result_restriction(
-                set([ci for ci in self.score_frame_["variable"][self.score_frame_["recommended"]]]))
+                set(
+                    [
+                        ci
+                        for ci in self.score_frame_["variable"][
+                            self.score_frame_["recommended"]
+                        ]
+                    ]
+                )
+            )
         cross_frame = vtreat_impl.limit_to_appropriate_columns(
             res=cross_frame, transform=self
         )
-        cross_frame, res_columns = vtreat_impl.back_to_orig_type_data_frame(cross_frame, orig_type)
+        cross_frame, res_columns = vtreat_impl.back_to_orig_type_data_frame(
+            cross_frame, orig_type
+        )
         self.last_result_columns = res_columns
-        if self.params_['retain_cross_plan']:
+        if self.params_["retain_cross_plan"]:
             self.cross_plan_ = cross_plan
             self.cross_rows_ = cross_rows
         else:
@@ -227,21 +285,21 @@ class BinomialOutcomeTreatment(vtreat_impl.VariableTreatment):
     """manage a treatment plan for a target outcome (binomial classification)"""
 
     def __init__(
-            self,
-            *,
-            var_list=None,
-            outcome_name=None,
-            outcome_target=True,
-            cols_to_copy=None,
-            params=None,
-            imputation_map=None,
+        self,
+        *,
+        var_list: Optional[Iterable[str]] = None,
+        outcome_name: Optional[str] = None,
+        outcome_target=None,
+        cols_to_copy: Optional[Iterable[str]] = None,
+        params: Optional[Dict[str, Any]] = None,
+        imputation_map: Optional[Dict[str, Any]] = None,
     ):
         """
 
-         :param var_list: list or touple of column names
+         :param var_list: list or tuple of column names, if empty all non outcome and copy columns are used
          :param outcome_name: name of column containing dependent variable
          :param outcome_target: value of outcome to consider "positive"
-         :param cols_to_copy: list or touple of column names
+         :param cols_to_copy: list or tuple of column names
          :param params: vtreat.vtreat_parameters()
          :param imputation_map: map of column names to custom missing imputation values or functions
         """
@@ -256,25 +314,46 @@ class BinomialOutcomeTreatment(vtreat_impl.VariableTreatment):
             imputation_map=imputation_map,
         )
 
-    def merge_params(self, p):
+    def merge_params(self, p: Optional[Dict[str, Any]]) -> Dict[str, Any]:
+        """
+        Merge user parameters, returns new parameters does not alter object.
+
+        :param p:
+        :return: merged parameters
+        """
+
         return vtreat_parameters(p)
 
     # noinspection PyPep8Naming
     def transform(self, X):
+        """
+        Apply transform to data.
+
+        :param X: data
+        :return: transformed data
+        """
+
         X, orig_type = vtreat_impl.ready_data_frame(X)
         self.check_column_names(X.columns)
         if self.last_fit_x_id_ is None:
             raise ValueError("called transform on not yet fit treatment")
-        if self.params_['check_for_duplicate_frames'] and (self.last_fit_x_id_ == vtreat.util.hash_data_frame(X)):
+        if self.params_["check_for_duplicate_frames"] and (
+            self.last_fit_x_id_ == vtreat.util.hash_data_frame(X)
+        ):
             if self.params_["error_on_duplicate_frames"]:
                 raise ValueError(
-                    "possibly called transform on same data used to fit\n" +
-                    "(this causes over-fit, please use fit_transform() instead)")
+                    "possibly called transform on same data used to fit\n"
+                    + "(this causes over-fit, please use fit_transform() instead)"
+                )
             warnings.warn(
-                "possibly called transform on same data used to fit\n" +
-                "(this causes over-fit, please use fit_transform() instead)")
+                "possibly called transform on same data used to fit\n"
+                + "(this causes over-fit, please use fit_transform() instead)"
+            )
         X = vtreat_impl.pre_prep_frame(
-            X, col_list=self.var_list_, cols_to_copy=self.cols_to_copy_, cat_cols=self.plan_['cat_list']
+            X,
+            col_list=self.plan_["num_list"] + self.plan_["cat_list"],
+            cols_to_copy=self.cols_to_copy_,
+            cat_cols=self.plan_["cat_list"],
         )
         res = vtreat_impl.perform_transform(x=X, transform=self, params=self.params_)
         res = vtreat_impl.limit_to_appropriate_columns(res=res, transform=self)
@@ -284,6 +363,15 @@ class BinomialOutcomeTreatment(vtreat_impl.VariableTreatment):
 
     # noinspection PyPep8Naming
     def fit_transform(self, X, y=None, **fit_params):
+        """
+        fit_transform data, this is the way to fit with cross methods.
+
+        :param X: explanatory values
+        :param y: dependent values
+        :param fit_params:
+        :return: transformed data
+        """
+
         X, orig_type = vtreat_impl.ready_data_frame(X)
         self.check_column_names(X.columns)
         if y is None:
@@ -294,7 +382,9 @@ class BinomialOutcomeTreatment(vtreat_impl.VariableTreatment):
             y = numpy.asarray(y)
             if (self.outcome_name_ is not None) and (self.outcome_name_ in X.columns):
                 if not numpy.all(X[self.outcome_name_] == y):
-                    raise ValueError(".fit_transform(X, y) called with y != X[outcome_name]")
+                    raise ValueError(
+                        ".fit_transform(X, y) called with y != X[outcome_name]"
+                    )
         if not X.shape[0] == len(y):
             raise ValueError("X.shape[0] should equal len(y)")
         y_mean = numpy.mean(y == self.outcome_target_)
@@ -302,7 +392,7 @@ class BinomialOutcomeTreatment(vtreat_impl.VariableTreatment):
             raise ValueError("y==outcome_target does not vary")
         cross_rows = None
         cross_plan = None
-        if self.params_['retain_cross_plan']:
+        if self.params_["retain_cross_plan"]:
             cross_rows = self.cross_rows_
             cross_plan = self.cross_plan_
         self.clear()
@@ -325,24 +415,25 @@ class BinomialOutcomeTreatment(vtreat_impl.VariableTreatment):
             params=self.params_,
             imputation_map=self.imputation_map_,
         )
-        res = vtreat_impl.perform_transform(x=X, transform=self, params=self.params_)
+        cross_frame = vtreat_impl.perform_transform(x=X, transform=self, params=self.params_)
         if (cross_plan is None) or (cross_rows != X.shape[0]):
             if cross_plan is not None:
-                warnings.warn("Number of rows different than previous fit with retain_cross_plan==True")
+                warnings.warn(
+                    "Number of rows different than previous fit with retain_cross_plan==True"
+                )
             cross_plan = self.params_["cross_validation_plan"].split_plan(
-                n_rows=X.shape[0], k_folds=self.params_["cross_validation_k"], data=X, y=y
+                n_rows=X.shape[0],
+                k_folds=self.params_["cross_validation_k"],
+                data=X,
+                y=y,
             )
             cross_rows = X.shape[0]
         # patch in cross-frame versions of complex columns such as impact
-        cross_frame = vtreat_impl.cross_patch_refit_y_aware_cols(
-            x=X, y=y, res=res, plan=self.plan_, cross_plan=cross_plan
+        vtreat_impl.cross_patch_refit_y_aware_cols(
+            x=X, y=y, res=cross_frame, plan=self.plan_, cross_plan=cross_plan
         )
-        cross_frame = vtreat_impl.cross_patch_user_y_aware_cols(
-            x=cross_frame,
-            y=y,
-            res=res,
-            params=self.params_,
-            cross_plan=cross_plan,
+        vtreat_impl.cross_patch_user_y_aware_cols(
+            x=cross_frame, y=y, res=cross_frame, params=self.params_, cross_plan=cross_plan,
         )
         # use cross_frame to compute variable effects
         self.score_frame_ = vtreat_impl.score_plan_variables(
@@ -352,17 +443,29 @@ class BinomialOutcomeTreatment(vtreat_impl.VariableTreatment):
             ),
             plan=self.plan_,
             params=self.params_,
-            is_classification=True
+            is_classification=True,
         )
-        if ("filter_to_recommended" in self.params_.keys()) and self.params_["filter_to_recommended"]:
+        if ("filter_to_recommended" in self.params_.keys()) and self.params_[
+            "filter_to_recommended"
+        ]:
             self.set_result_restriction(
-                set([ci for ci in self.score_frame_["variable"][self.score_frame_["recommended"]]]))
+                set(
+                    [
+                        ci
+                        for ci in self.score_frame_["variable"][
+                            self.score_frame_["recommended"]
+                        ]
+                    ]
+                )
+            )
         cross_frame = vtreat_impl.limit_to_appropriate_columns(
             res=cross_frame, transform=self
         )
-        cross_frame, res_columns = vtreat_impl.back_to_orig_type_data_frame(cross_frame, orig_type)
+        cross_frame, res_columns = vtreat_impl.back_to_orig_type_data_frame(
+            cross_frame, orig_type
+        )
         self.last_result_columns = res_columns
-        if self.params_['retain_cross_plan']:
+        if self.params_["retain_cross_plan"]:
             self.cross_plan_ = cross_plan
             self.cross_rows_ = cross_rows
         else:
@@ -375,19 +478,19 @@ class MultinomialOutcomeTreatment(vtreat_impl.VariableTreatment):
     """manage a treatment plan for a set of outcomes (multinomial classification)"""
 
     def __init__(
-            self,
-            *,
-            var_list=None,
-            outcome_name=None,
-            cols_to_copy=None,
-            params=None,
-            imputation_map=None,
+        self,
+        *,
+        var_list: Optional[Iterable[str]] = None,
+        outcome_name: Optional[str] = None,
+        cols_to_copy: Optional[Iterable[str]] = None,
+        params: Optional[Dict[str, Any]] = None,
+        imputation_map: Optional[Dict[str, Any]] = None,
     ):
         """
 
-         :param var_list: list or touple of column names
+         :param var_list: list or tuple of column names, if empty all non outcome and copy columns are used
          :param outcome_name: name of column containing dependent variable
-         :param cols_to_copy: list or touple of column names
+         :param cols_to_copy: list or tuple of column names
          :param params: vtreat.vtreat_parameters()
          :param imputation_map: map of column names to custom missing imputation values or functions
         """
@@ -403,25 +506,46 @@ class MultinomialOutcomeTreatment(vtreat_impl.VariableTreatment):
         )
         self.outcomes_ = None
 
-    def merge_params(self, p):
+    def merge_params(self, p: Optional[Dict[str, Any]]) -> Dict[str, Any]:
+        """
+        Merge user parameters, returns new parameters does not alter object.
+
+        :param p:
+        :return: merged parameters
+        """
+
         return vtreat_parameters(p)
 
     # noinspection PyPep8Naming
     def transform(self, X):
+        """
+        Apply transform to data.
+
+        :param X: data
+        :return: transformed data
+        """
+
         X, orig_type = vtreat_impl.ready_data_frame(X)
         self.check_column_names(X.columns)
         if self.last_fit_x_id_ is None:
             raise ValueError("called transform on not yet fit treatment")
-        if self.params_['check_for_duplicate_frames'] and (self.last_fit_x_id_ == vtreat.util.hash_data_frame(X)):
+        if self.params_["check_for_duplicate_frames"] and (
+            self.last_fit_x_id_ == vtreat.util.hash_data_frame(X)
+        ):
             if self.params_["error_on_duplicate_frames"]:
                 raise ValueError(
-                    "possibly called transform on same data used to fit\n" +
-                    "(this causes over-fit, please use fit_transform() instead)")
+                    "possibly called transform on same data used to fit\n"
+                    + "(this causes over-fit, please use fit_transform() instead)"
+                )
             warnings.warn(
-                "possibly called transform on same data used to fit\n" +
-                "(this causes over-fit, please use fit_transform() instead)")
+                "possibly called transform on same data used to fit\n"
+                + "(this causes over-fit, please use fit_transform() instead)"
+            )
         X = vtreat_impl.pre_prep_frame(
-            X, col_list=self.var_list_, cols_to_copy=self.cols_to_copy_, cat_cols=self.plan_['cat_list']
+            X,
+            col_list=self.plan_["num_list"] + self.plan_["cat_list"],
+            cols_to_copy=self.cols_to_copy_,
+            cat_cols=self.plan_["cat_list"],
         )
         res = vtreat_impl.perform_transform(x=X, transform=self, params=self.params_)
         res = vtreat_impl.limit_to_appropriate_columns(res=res, transform=self)
@@ -431,6 +555,15 @@ class MultinomialOutcomeTreatment(vtreat_impl.VariableTreatment):
 
     # noinspection PyPep8Naming
     def fit_transform(self, X, y=None, **fit_params):
+        """
+        fit_transform data, this is the way to fit with cross methods.
+
+        :param X: explanatory values
+        :param y: dependent values
+        :param fit_params:
+        :return: transformed data
+        """
+
         X, orig_type = vtreat_impl.ready_data_frame(X)
         self.check_column_names(X.columns)
         if y is None:
@@ -441,14 +574,16 @@ class MultinomialOutcomeTreatment(vtreat_impl.VariableTreatment):
             y = numpy.asarray(y)
             if (self.outcome_name_ is not None) and (self.outcome_name_ in X.columns):
                 if not numpy.all(X[self.outcome_name_] == y):
-                    raise ValueError(".fit_transform(X, y) called with y != X[outcome_name]")
+                    raise ValueError(
+                        ".fit_transform(X, y) called with y != X[outcome_name]"
+                    )
         if not X.shape[0] == len(y):
             raise ValueError("X.shape[0] should equal len(y)")
         if len(numpy.unique(y)) <= 1:
             raise ValueError("y must take on at least 2 values")
         cross_rows = None
         cross_plan = None
-        if self.params_['retain_cross_plan']:
+        if self.params_["retain_cross_plan"]:
             cross_rows = self.cross_rows_
             cross_plan = self.cross_plan_
         self.clear()
@@ -471,33 +606,35 @@ class MultinomialOutcomeTreatment(vtreat_impl.VariableTreatment):
             params=self.params_,
             imputation_map=self.imputation_map_,
         )
-        res = vtreat_impl.perform_transform(x=X, transform=self, params=self.params_)
+        cross_frame = vtreat_impl.perform_transform(x=X, transform=self, params=self.params_)
         if (cross_plan is None) or (cross_rows != X.shape[0]):
             if cross_plan is not None:
-                warnings.warn("Number of rows different than previous fit with retain_cross_plan==True")
+                warnings.warn(
+                    "Number of rows different than previous fit with retain_cross_plan==True"
+                )
             cross_plan = self.params_["cross_validation_plan"].split_plan(
-                n_rows=X.shape[0], k_folds=self.params_["cross_validation_k"], data=X, y=y
+                n_rows=X.shape[0],
+                k_folds=self.params_["cross_validation_k"],
+                data=X,
+                y=y,
             )
             cross_rows = X.shape[0]
-        cross_frame = vtreat_impl.cross_patch_refit_y_aware_cols(
-            x=X, y=y, res=res, plan=self.plan_, cross_plan=cross_plan
+        vtreat_impl.cross_patch_refit_y_aware_cols(
+            x=X, y=y, res=cross_frame, plan=self.plan_, cross_plan=cross_plan
         )
-        cross_frame = vtreat_impl.cross_patch_user_y_aware_cols(
-            x=cross_frame,
-            y=y,
-            res=res,
-            params=self.params_,
-            cross_plan=cross_plan,
+        vtreat_impl.cross_patch_user_y_aware_cols(
+            x=cross_frame, y=y, res=cross_frame, params=self.params_, cross_plan=cross_plan,
         )
         # use cross_frame to compute variable effects
 
         def si(oi):
+            """score i-th outcome group"""
             sf = vtreat_impl.score_plan_variables(
                 cross_frame=cross_frame,
                 outcome=numpy.asarray(numpy.asarray(y) == oi, dtype=float),
                 plan=self.plan_,
                 params=self.params_,
-                is_classification=True
+                is_classification=True,
             )
             sf["outcome_target"] = oi
             return sf
@@ -505,15 +642,27 @@ class MultinomialOutcomeTreatment(vtreat_impl.VariableTreatment):
         score_frames = [si(oi) for oi in self.outcomes_]
         self.score_frame_ = pandas.concat(score_frames, axis=0)
         self.score_frame_.reset_index(inplace=True, drop=True)
-        if ("filter_to_recommended" in self.params_.keys()) and self.params_["filter_to_recommended"]:
+        if ("filter_to_recommended" in self.params_.keys()) and self.params_[
+            "filter_to_recommended"
+        ]:
             self.set_result_restriction(
-                set([ci for ci in self.score_frame_["variable"][self.score_frame_["recommended"]]]))
+                set(
+                    [
+                        ci
+                        for ci in self.score_frame_["variable"][
+                            self.score_frame_["recommended"]
+                        ]
+                    ]
+                )
+            )
         cross_frame = vtreat_impl.limit_to_appropriate_columns(
             res=cross_frame, transform=self
         )
-        cross_frame, res_columns = vtreat_impl.back_to_orig_type_data_frame(cross_frame, orig_type)
+        cross_frame, res_columns = vtreat_impl.back_to_orig_type_data_frame(
+            cross_frame, orig_type
+        )
         self.last_result_columns = res_columns
-        if self.params_['retain_cross_plan']:
+        if self.params_["retain_cross_plan"]:
             self.cross_plan_ = cross_plan
             self.cross_rows_ = cross_rows
         else:
@@ -525,16 +674,18 @@ class MultinomialOutcomeTreatment(vtreat_impl.VariableTreatment):
 class UnsupervisedTreatment(vtreat_impl.VariableTreatment):
     """manage an unsupervised treatment plan"""
 
-    def __init__(self,
-                 *,
-                 var_list=None,
-                 cols_to_copy=None,
-                 params=None,
-                 imputation_map=None):
+    def __init__(
+            self,
+            *,
+            var_list: Optional[Iterable[str]] = None,
+            cols_to_copy: Optional[Iterable[str]] = None,
+            params: Optional[Dict[str, Any]] = None,
+            imputation_map: Optional[Dict[str, Any]] = None,
+    ):
         """
 
-        :param var_list: list or touple of column names
-        :param cols_to_copy: list or touple of column names
+        :param var_list: list or tuple of column names, if empty all non copy columns are used
+        :param cols_to_copy: list or tuple of column names
         :param params: vtreat.unsupervised_parameters()
         :param imputation_map: map of column names to custom missing imputation values or functions
         """
@@ -548,17 +699,34 @@ class UnsupervisedTreatment(vtreat_impl.VariableTreatment):
             imputation_map=imputation_map,
         )
 
-    def merge_params(self, p):
+    def merge_params(self, p: Optional[Dict[str, Any]]) -> Dict[str, Any]:
+        """
+        Merge user parameters, returns new parameters does not alter object.
+
+        :param p:
+        :return: merged parameters
+        """
+
         return unsupervised_parameters(p)
 
     # noinspection PyPep8Naming
     def transform(self, X):
+        """
+        Apply transform to data.
+
+        :param X: data
+        :return: transformed data
+        """
+
         X, orig_type = vtreat_impl.ready_data_frame(X)
         self.check_column_names(X.columns)
         if self.last_fit_x_id_ is None:
             raise ValueError("called transform on not yet fit treatment")
         X = vtreat_impl.pre_prep_frame(
-            X, col_list=self.var_list_, cols_to_copy=self.cols_to_copy_, cat_cols=self.plan_['cat_list']
+            X,
+            col_list=self.plan_["num_list"] + self.plan_["cat_list"],
+            cols_to_copy=self.cols_to_copy_,
+            cat_cols=self.plan_["cat_list"],
         )
         res = vtreat_impl.perform_transform(x=X, transform=self, params=self.params_)
         res = vtreat_impl.limit_to_appropriate_columns(res=res, transform=self)
@@ -568,6 +736,15 @@ class UnsupervisedTreatment(vtreat_impl.VariableTreatment):
 
     # noinspection PyPep8Naming
     def fit_transform(self, X, y=None, **fit_params):
+        """
+        fit_transform data.
+
+        :param X: explanatory values
+        :param y: dependent values
+        :param fit_params:
+        :return: transformed data
+        """
+
         X, orig_type = vtreat_impl.ready_data_frame(X)
         self.check_column_names(X.columns)
         if y is not None:
@@ -589,9 +766,19 @@ class UnsupervisedTreatment(vtreat_impl.VariableTreatment):
         self.score_frame_ = vtreat_impl.pseudo_score_plan_variables(
             cross_frame=res, plan=self.plan_, params=self.params_
         )
-        if ("filter_to_recommended" in self.params_.keys()) and self.params_["filter_to_recommended"]:
+        if ("filter_to_recommended" in self.params_.keys()) and self.params_[
+            "filter_to_recommended"
+        ]:
             self.set_result_restriction(
-                set([ci for ci in self.score_frame_["variable"][self.score_frame_["recommended"]]]))
+                set(
+                    [
+                        ci
+                        for ci in self.score_frame_["variable"][
+                            self.score_frame_["recommended"]
+                        ]
+                    ]
+                )
+            )
         res = vtreat_impl.limit_to_appropriate_columns(res=res, transform=self)
         res, res_columns = vtreat_impl.back_to_orig_type_data_frame(res, orig_type)
         self.last_result_columns = res_columns
